from datetime import datetime

from enum import Enum


class FilingItem(Enum):
    ITEM_1 = "Item 1"
    ITEM_1A = "Item 1A"
    ITEM_1B = "Item 1B"
    ITEM_2 = "Item 2"
    ITEM_3 = "Item 3"
    ITEM_4 = "Item 4"
    ITEM_5 = "Item 5"
    ITEM_6 = "Item 6"
    ITEM_7 = "Item 7"
    ITEM_7A = "Item 7A"
    ITEM_8 = "Item 8"
    ITEM_9 = "Item 9"
    ITEM_9A = "Item 9A"
    ITEM_9B = "Item 9B"
    ITEM_10 = "Item 10"
    ITEM_11 = "Item 11"
    ITEM_12 = "Item 12"
    ITEM_13 = "Item 13"
    ITEM_14 = "Item 14"
    ITEM_15 = "Item 15"
    ITEM_16 = "Item 16"


def filter_filings(filings, form: str, year: int):
    """
    Filter filings by form and year.

    :param filings: List of EntityFilings.
    :param form: The form to filter by.
    :param year: The year to filter by.
    :return: The filtered EntityFiling.
    """
    for filing in filings:
        report_date_str = filing.report_date
        report_date = datetime.strptime(report_date_str, "%Y-%m-%d")
        if filing.form == form and report_date.year == year:
            return filing

    raise ValueError(f"No {form} filing found for the year {year}.")



from typing import List


def generate_dataset() -> List[dict]:
    return [
        {
            "type": "function",
            "function": {
                "name": 'generate_dataset',
                "description": 'This function generates a list of dataset items. Each dataset item has a question, answer, and context',
                "parameters": {
                    "type": 'object',
                    "properties": {
                        "dataset_items": {
                            "type": 'array',
                            "items": {
                                "type": 'object',
                                "properties": {
                                    "question": {
                                        "type": 'string',
                                        "description": 'The generated question.',
                                    },
                                    "answer": {
                                        "type": 'string',
                                        "description": 'The generated answer.',
                                    },
                                    "context": {
                                        "type": 'string',
                                        "description": "The context that the question and answer are generated from.",
                                    },
                                },
                                "required": ['question', 'answer', 'context'],
                            },
                        }
                    },
                    "required": ['dataset_items'],
                },
            }
        }
    ]



default_prompt = """
You are an expert at understanding financial documents and generating datasets. 
Your primary role is to generate question and ground truth answer pairs based on the provided financial text. 
The types of texts you will be working with include 10-Ks, 10-Qs, earnings call transcripts, PDFs, and other financial documents.

When generating questions and answers, you MUST follow these rules:
1. Your ground truth answers must be directly derived from the content within the provided text. Do not make up, hallucinate, or generate answers that are not explicitly supported by the given text.
2. The question should be fully answerable from information present in given context.
3. Make sure the question is clear and unambiguous.
4. Phrases that reference the underlying text like ’based on the provided context’, ’according to the context’, "in the text", etc., are not allowed to appear in the question because the answerer of the question will not have access to the context.
5. Include the relevant 'context' paragraph from which you generated each question and ground truth answer pair. The 'context' paragraph MUST contain the specific information that supports the ground truth answer.
6. If the provided text does not contain sufficient information to generate a question-answer pair, do not attempt to create one.
7. The answer must use the information provided in the context.
8 Do not just copy words from the context. Answer the question in your own words.
9. Your responses should be in the following format:
   Question: [Generated question]
   Answer: [Ground truth answer]
   Context: [Relevant paragraph from the text that supports the ground truth answer]

Important: your primary objective is to create accurate, grounded, and contextually relevant question-answer pairs while strictly avoiding any fabrication or speculation.
"""


from openai import OpenAI
from tenacity import retry, wait_random_exponential, stop_after_attempt

client = OpenAI()


@retry(wait=wait_random_exponential(multiplier=1, max=40), stop=stop_after_attempt(3))
def chat_completion_request(model: str, messages, tools=None, tool_choice=None):
    try:
        response = client.chat.completions.create(
            model=model,
            messages=messages,
            tools=tools,
            tool_choice=tool_choice,
        )
        return response
    except Exception as e:
        print("Unable to generate ChatCompletion response")
        print(f"Exception: {e}")
        return e
